#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Movable.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 1, "SCORE");
    PrintS(Vram + 7, "HI-SCORE");
    PrintS(Vram + 16, "STAGE");
    PrintByteNumber2(Vram + VramWidth + 18, CurrentStage + 1);
    PrintS(Vram+ 22, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + 27;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                byte c;
                c = Char_Remain;
                repeat (2) {
                    repeat (2) {
                        *pVram = c; ++pVram;
                        ++c;
                    }
                    pVram += VramWidth - 2;
                }
                pVram += 2 - VramWidth * 2;
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramWidth, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramWidth + 9, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramWidth + 23, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramWidth * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramWidth - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramWidth * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramWidth * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	L
        0x28, 0x1f, 0x1c, 0x1c, 
        0x28, 0x1f, 0x1c, 0x1c, 
        0x28, 0x1f, 0x1c, 0x1c, 
        0x20, 0x21, 0x21, 0x1d, 
        //	I
        0x20, 0x29, 0x23, 0x1d, 
        0x1c, 0x28, 0x1f, 0x1c, 
        0x1c, 0x28, 0x1f, 0x1c, 
        0x20, 0x21, 0x21, 0x1d, 
        //	F
        0x28, 0x23, 0x21, 0x1d, 
        0x28, 0x27, 0x26, 0x1e, 
        0x28, 0x1f, 0x1c, 0x1c, 
        0x20, 0x1d, 0x1c, 0x1c, 
        //	T
        0x20, 0x29, 0x23, 0x1d, 
        0x1c, 0x28, 0x1f, 0x1c, 
        0x1c, 0x28, 0x1f, 0x1c, 
        0x1c, 0x20, 0x1d, 0x1c, 
   };

    ClearScreen(); 
    HideAllSprites();
    UpdateSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramWidth * 7 + (32 - 4 * 4) / 2;
        p = TitleBytes;
        repeat (4) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p; ++pVram;
                    ++p;
                }
                pVram += VramWidth - 4;
            }
            pVram += 4 - VramWidth * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 7, "PUSH TRIGGER BUTTON");
    PrintS(Vram + VramWidth * 23 + 20, "INUFUTO 2022");
}